# App Settings
from django.conf import settings
from django import forms
from django.db.models import TextChoices
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext_lazy as _

from siteconfigs.config import SiteConfigBaseClass
from siteconfigs.models import SiteConfigModel

class SupportedPaymentProcessor(TextChoices):
    PROMO_CODE_BASE = ("base.PaymentProcessorBase", _("Default Processor"))
    AUTHORIZE_NET = ("authorizenet.AuthorizeNetProcessor", _("Authorize.Net"))


class PaymentProcessorForm(forms.Form):
    payment_processor = forms.CharField(label=_("Payment Processor"), widget=forms.Select(choices=SupportedPaymentProcessor.choices))


class PaymentProcessorSiteSelectForm(PaymentProcessorForm):
    site = forms.CharField(label=_("Site"))

    def __init__(self, *args, **kwargs):
        self.fields['site'].widget = forms.Select(choices=[(site.pk, site.domain) for site in Site.objects.all()])


class PaymentProcessorSiteConfig(SiteConfigBaseClass):
    label = _("Promo Code Processor")
    default = {"payment_processor": "base.PaymentProcessorBase"}
    form_class = PaymentProcessorForm
    key = ""
    instance = None

    def __init__(self, site=None):
        if site is None:
            site = Site.objects.get_current()

        self.key = ".".join([__name__, __class__.__name__])
        super().__init__(site, self.key)
        
        if not self.instance:
            self.default['payment_processor'] = VENDOR_PAYMENT_PROCESSOR

    def get_form(self):
        return self.form_class(initial=self.get_initials())

    def get_initials(self):
        if self.instance:
            return {'payment_processor': [choice for choice in SupportedPaymentProcessor.choices if choice[0] == self.instance.value['payment_processor']][0]}
        return {'payment_processor': SupportedPaymentProcessor.choices[0]}

    def get_selected_processor(self):
        if self.instance:
            return [choice for choice in SupportedPaymentProcessor.choices if choice[0] == self.instance.value['payment_processor']][0]
        return SupportedPaymentProcessor.choices[0]  # Return Default Processors


class PaymentProcessorSiteSelectSiteConfig(PaymentProcessorSiteConfig):
    label = _("Promo Code Processor")
    default = {"payment_processor": "base.PromoProcessorBase"}
    form_class = PaymentProcessorSiteSelectForm
    key = ""
    instance = None

    def get_initials(self):
        initial = super().get_initials()
        initial['site'] = (self.site.pk, self.site.domain)
        return initial


VENDOR_PRODUCT_MODEL = getattr(settings, "VENDOR_PRODUCT_MODEL", "vendor.Product")

VENDOR_PAYMENT_PROCESSOR = getattr(settings, "VENDOR_PAYMENT_PROCESSOR", "dummy.DummyProcessor")

DEFAULT_CURRENCY = getattr(settings, "DEFAULT_CURRENCY", "usd")

AVAILABLE_CURRENCIES = getattr(settings, "AVAILABLE_CURRENCIES", {'usd': _('USD Dollars')})

VENDOR_STATE = getattr(settings, "VENDOR_STATE", "DEBUG")

# Encryption settings
VENDOR_DATA_ENCODER = getattr(settings, "VENDOR_DATA_ENCODER", "vendor.encrypt.cleartext")



from django.utils.module_loading import import_string
from django.core.exceptions import ObjectDoesNotExist

from vendor.config import PaymentProcessorSiteConfig
from vendor.processors.authorizenet import AuthorizeNetProcessor
from vendor.processors.base import PaymentProcessorBase
from vendor.processors.dummy import DummyProcessor
from siteconfigs.models import SiteConfigModel


def get_site_payment_processor(site):
    site_processor = PaymentProcessorSiteConfig(site)
    return import_string(f"vendor.processors.{site_processor.get_key_value('payment_processor')}")



# Generated by Django 3.1.3 on 2020-12-03 21:07

import django.contrib.sites.managers
from django.db import migrations
import django.db.models.manager
import vendor.models.offer


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0016_auto_20201203_2011'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='offer',
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
                ('on_site_active', vendor.models.offer.ActiveCurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 3.1.3 on 2020-12-02 17:59

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0012_auto_20201123_1848'),
    ]

    operations = [
        migrations.AddField(
            model_name='payment',
            name='uuid',
            field=models.UUIDField(editable=True, unique=False, verbose_name='UUID', null=True),
        ),
        migrations.AddField(
            model_name='receipt',
            name='uuid',
            field=models.UUIDField(editable=True, unique=False, verbose_name='UUID', null=True),
        ),
        migrations.AlterField(
            model_name='offer',
            name='terms',
            field=models.IntegerField(choices=[(100, 'Subscription'), (101, 'Monthly Subscription'), (103, 'Quarterly Subscription'), (106, 'Semi-Annual Subscription'), (112, 'Annual Subscription'), (200, 'Perpetual'), (220, 'One-Time Use')], default=0, verbose_name='Terms'),
        ),
    ]
